# -*- coding: utf-8 -*-

import importlib
import logging
import logging.config
import os
import re
import sys
from collections.abc import Mapping
from logging import Logger
from pathlib import Path
from typing import Callable, Dict, List, Tuple

import yaml
from injector import Binder, Injector, Key, singleton

from invoker.utils import AliasDict, split_qualified_name

LOGGER = logging.getLogger(__name__)

InvokeConfig = Key('InvokeConfig')  # type: AliasDict
InvokeOptions = Key('InvokeOptions')  # type: Dict


class InvokerContext():

    logger = LOGGER.getChild('InvokerContext')  # type: Logger

    IS_ALREADY_LOADED_LOGGING = False

    @classmethod
    def set_logging_config(cls, logging_config_file: Path):

        if cls.IS_ALREADY_LOADED_LOGGING:
            cls.logger.warning(
                'already initilize logging configuration. skip.')

        else:
            cls.logger.info('initilize logging configuration. start')
            with logging_config_file.open('r', encoding='utf-8') as f:
                config = yaml.load(f)

            logging.config.dictConfig(config)
            cls.IS_ALREADY_LOADED_LOGGING = True
            cls.logger.info(
                'initilize logging configuration. end: \n%s', config)

    def __init__(self, config_file_list: List[Path],
                 logging_config_path: Path = None):

        if logging_config_path:
            self.set_logging_config(logging_config_path)

        # logging default setting.
        config = AliasDict({})
        for c_file in config_file_list:
            cl = AliasDict.load_from_yaml(c_file)
            config.update(cl)

        self.app_config = config
        self.injector = Injector(modules=[self._injector_bind])  # type: Injector
        self.invoke_options = None

    def _injector_bind(self, binder: Binder):
        binder.bind(InvokeConfig, to=self.app_config, scope=singleton)

    def invoke(self, invoke_options: Dict):

        self.invoke_options = invoke_options
        self.injector.binder.bind(InvokeOptions, to=self.invoke_options, scope=singleton)

        _package, _callable = split_qualified_name(invoke_options['invoke'])
        self.logger.debug('calleble: %s, package: %s', _callable, _package)

        self.logger.debug('cwd: %s', os.getcwd())
        sys.path.append(os.getcwd())
        self.logger.debug('sys.path: \n %s', '\n '.join(sys.path))

        _func = getattr(importlib.import_module(_package), _callable)  # type: Callable

        kwargs = invoke_options.get('args', {})
        try:
            return self._invoke(_func, args=[], kwargs=kwargs)
        except Exception as e:
            self.logger.exception('invoke function internal error.')
            sys.exit(10)

    def _invoke(self, func: Callable, args: Tuple, kwargs: Dict) -> any:

        self.logger.info('func: %s  args: %s, kwargs: %s', func, args, kwargs)
        return self.injector.call_with_injection(func,
                                                 args=args,
                                                 kwargs=kwargs)



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(
    name="invoker",
    version="1.0.2",
    author='636',
    author_email='zx6r.636a1@gmail.com',
    description='Invoker utility for DI(Injector) and context configuration.',
    url='https://github.com/636/python3_invoker',
    packages=find_packages(),
    install_requires=["injector==0.14.1", "pyyaml>=4.2b1"],
    extras_require={
        "develop": ["autopep8", "pep8"]
    },
    entry_points={
        "console_scripts": [
            "invoker = invoker.cmd:execute"
        ],
    },
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6'
    ],
)



# -*- coding: utf-8 -*-

from unittest import TestCase
import logging

from invoker.utils import AliasDict
from logging import StreamHandler

LOGGER = logging.getLogger(__name__)
sh = StreamHandler()
sh.setLevel(logging.DEBUG)
LOGGER.addHandler(sh)
LOGGER.setLevel(logging.DEBUG)


class TestAliasDict(TestCase):

    logger = LOGGER.getChild('TestAliasDict')

    def test_replace(self):

        config = AliasDict({
            'test': {
                'base': 'aaa/bbb',
                'concat': "${test.base}/ccc"
            }
        })

        self.logger.debug(config['test.concat'])
        self.assertEqual(config.get('test.concat'), 'aaa/bbb/ccc')
